@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """
 ----------------
@@ -27,43 +22,24 @@ Lark already `supports loading grammars
 <https://lark-parser.readthedocs.io/en/latest/nearley.html>`_
 from `nearley.js <https://nearley.js.org/>`_, so you may not have to write
 your own at all.
-
-Note that as Lark is at version 0.x, this module *may* break API compatibility
-in minor releases if supporting the latest version of Lark would otherwise be
-infeasible.  We may also be quite aggressive in bumping the minimum version of
-Lark, unless someone volunteers to either fund or do the maintainence.
 """
 
-from inspect import getfullargspec
+from inspect import signature
 from typing import Dict, Optional
 
-import attr
 import lark
 from lark.grammar import NonTerminal, Terminal
 
+from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.conjecture.utils import calc_label_from_name
-from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import check_type
-from hypothesis.strategies._internal import SearchStrategy, core as st
+from hypothesis.strategies._internal.regex import IncompatibleWithAlphabet
+from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 
 __all__ = ["from_lark"]
 
 
-@attr.s()
-class DrawState:
-    """Tracks state of a single draw from a lark grammar.
-
-    Currently just wraps a list of tokens that will be emitted at the
-    end, but as we support more sophisticated parsers this will need
-    to track more state for e.g. indentation level.
-    """
-
-    # The text output so far as a list of string tokens resulting from
-    # each draw to a non-terminal.
-    result = attr.ib(default=attr.Factory(list))
-
-
 def get_terminal_names(terminals, rules, ignore_names):
     """Get names of all terminals in the grammar.
 
@@ -78,13 +54,13 @@ def get_terminal_names(terminals, rules, ignore_names):
     return names
 
 
-class LarkStrategy(SearchStrategy):
+class LarkStrategy(st.SearchStrategy):
     """Low-level strategy implementation wrapping a Lark grammar.
 
     See ``from_lark`` for details.
     """
 
-    def __init__(self, grammar, start, explicit):
+    def __init__(self, grammar, start, explicit, alphabet):
         assert isinstance(grammar, lark.lark.Lark)
         if start is None:
             start = grammar.options.start
@@ -95,7 +71,7 @@ class LarkStrategy(SearchStrategy):
         # This is a total hack, but working around the changes is a nicer user
         # experience than breaking for anyone who doesn't instantly update their
         # installation of Lark alongside Hypothesis.
-        compile_args = getfullargspec(grammar.grammar.compile).args
+        compile_args = signature(grammar.grammar.compile).parameters
         if "terminals_to_keep" in compile_args:
             terminals, rules, ignore_names = grammar.grammar.compile(start, ())
         elif "start" in compile_args:  # pragma: no cover
@@ -111,68 +87,88 @@ class LarkStrategy(SearchStrategy):
             t = r.origin
             self.names_to_symbols[t.name] = t
 
+        disallowed = set()
+        self.terminal_strategies = {}
         for t in terminals:
             self.names_to_symbols[t.name] = Terminal(t.name)
-
-        self.start = st.sampled_from([self.names_to_symbols[s] for s in start])
+            s = st.from_regex(t.pattern.to_regexp(), fullmatch=True, alphabet=alphabet)
+            try:
+                s.validate()
+            except IncompatibleWithAlphabet:
+                disallowed.add(t.name)
+            else:
+                self.terminal_strategies[t.name] = s
 
         self.ignored_symbols = tuple(self.names_to_symbols[n] for n in ignore_names)
 
-        self.terminal_strategies = {
-            t.name: st.from_regex(t.pattern.to_regexp(), fullmatch=True)
-            for t in terminals
-        }
-        unknown_explicit = set(explicit) - get_terminal_names(
-            terminals, rules, ignore_names
-        )
-        if unknown_explicit:
+        all_terminals = get_terminal_names(terminals, rules, ignore_names)
+        if unknown_explicit := sorted(set(explicit) - all_terminals):
             raise InvalidArgument(
-                "The following arguments were passed as explicit_strategies, "
-                "but there is no such terminal production in this grammar: %r"
-                % (sorted(unknown_explicit),)
+                "The following arguments were passed as explicit_strategies, but "
+                f"there is no {unknown_explicit} terminal production in this grammar."
+            )
+        if missing_declared := sorted(
+            all_terminals - {t.name for t in terminals} - set(explicit)
+        ):
+            raise InvalidArgument(
+                f"Undefined terminal{'s' * (len(missing_declared) > 1)} "
+                f"{sorted(missing_declared)!r}. Generation does not currently "
+                "support use of %declare unless you pass `explicit`, a dict of "
+                f"names-to-strategies, such as `{{{missing_declared[0]!r}: "
+                'st.just("")}}`'
             )
         self.terminal_strategies.update(explicit)
 
         nonterminals = {}
 
         for rule in rules:
-            nonterminals.setdefault(rule.origin.name, []).append(tuple(rule.expansion))
+            if disallowed.isdisjoint(r.name for r in rule.expansion):
+                nonterminals.setdefault(rule.origin.name, []).append(
+                    tuple(rule.expansion)
+                )
 
-        for v in nonterminals.values():
-            v.sort(key=len)
+        allowed_rules = {*self.terminal_strategies, *nonterminals}
+        while dict(nonterminals) != (
+            nonterminals := {
+                k: clean
+                for k, v in nonterminals.items()
+                if (clean := [x for x in v if all(r.name in allowed_rules for r in x)])
+            }
+        ):
+            allowed_rules = {*self.terminal_strategies, *nonterminals}
+
+        if set(start).isdisjoint(allowed_rules):
+            raise InvalidArgument(
+                f"No start rule {tuple(start)} is allowed by {alphabet=}"
+            )
+        self.start = st.sampled_from(
+            [self.names_to_symbols[s] for s in start if s in allowed_rules]
+        )
 
         self.nonterminal_strategies = {
-            k: st.sampled_from(v) for k, v in nonterminals.items()
+            k: st.sampled_from(sorted(v, key=len)) for k, v in nonterminals.items()
         }
 
         self.__rule_labels = {}
 
     def do_draw(self, data):
-        state = DrawState()
+        state = []
         start = data.draw(self.start)
         self.draw_symbol(data, start, state)
-        return "".join(state.result)
+        return "".join(state)
 
     def rule_label(self, name):
         try:
             return self.__rule_labels[name]
         except KeyError:
             return self.__rule_labels.setdefault(
-                name, calc_label_from_name("LARK:%s" % (name,))
+                name, calc_label_from_name(f"LARK:{name}")
             )
 
     def draw_symbol(self, data, symbol, draw_state):
         if isinstance(symbol, Terminal):
-            try:
-                strategy = self.terminal_strategies[symbol.name]
-            except KeyError:
-                raise InvalidArgument(
-                    "Undefined terminal %r. Generation does not currently support "
-                    "use of %%declare unless you pass `explicit`, a dict of "
-                    'names-to-strategies, such as `{%r: st.just("")}`'
-                    % (symbol.name, symbol.name)
-                )
-            draw_state.result.append(data.draw(strategy))
+            strategy = self.terminal_strategies[symbol.name]
+            draw_state.append(data.draw(strategy))
         else:
             assert isinstance(symbol, NonTerminal)
             data.start_example(self.rule_label(symbol.name))
@@ -183,7 +179,7 @@ class LarkStrategy(SearchStrategy):
             data.stop_example()
 
     def gen_ignore(self, data, draw_state):
-        if self.ignored_symbols and data.draw_bits(2) == 3:
+        if self.ignored_symbols and data.draw_boolean(1 / 4):
             emit = data.draw(st.sampled_from(self.ignored_symbols))
             self.draw_symbol(data, emit, draw_state)
 
@@ -199,14 +195,14 @@ def check_explicit(name):
     return inner
 
 
-@st.cacheable
-@st.defines_strategy(force_reusable_values=True)
-@deprecated_posargs
+@cacheable
+@defines_strategy(force_reusable_values=True)
 def from_lark(
     grammar: lark.lark.Lark,
     *,
     start: Optional[str] = None,
     explicit: Optional[Dict[str, st.SearchStrategy[str]]] = None,
+    alphabet: st.SearchStrategy[str] = st.characters(codec="utf-8"),
 ) -> st.SearchStrategy[str]:
     """A strategy for strings accepted by the given context-free grammar.
 
@@ -237,7 +233,7 @@ def from_lark(
     else:
         check_type(dict, explicit, "explicit")
         explicit = {
-            k: v.map(check_explicit("explicit[%r]=%r" % (k, v)))
+            k: v.map(check_explicit(f"explicit[{k!r}]={v!r}"))
             for k, v in explicit.items()
         }
-    return LarkStrategy(grammar, start, explicit)
+    return LarkStrategy(grammar, start, explicit, alphabet)
