@@ -8,16 +8,8 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
-from hypothesis import (
-    HealthCheck,
-    Phase,
-    Verbosity,
-    given,
-    settings as Settings,
-    strategies as st,
-)
+from hypothesis import HealthCheck, Phase, Verbosity, given, settings as Settings
 from hypothesis.errors import Found, NoSuchExample, Unsatisfiable
-from hypothesis.internal.conjecture.data import ConjectureData, StopTest
 from hypothesis.internal.reflection import get_pretty_function_description
 
 from tests.common.utils import no_shrink
@@ -30,15 +22,20 @@ class Timeout(BaseException):
 
 
 def minimal(definition, condition=lambda x: True, settings=None, timeout_after=10):
+    definition.validate()
+    runtime = None
+    result = None
+
     def wrapped_condition(x):
+        nonlocal runtime
         if timeout_after is not None:
             if runtime:
-                runtime[0] += TIME_INCREMENT
-                if runtime[0] >= timeout_after:
-                    raise Timeout()
+                runtime += TIME_INCREMENT
+                if runtime >= timeout_after:
+                    raise Timeout
         result = condition(x)
         if result and not runtime:
-            runtime.append(0.0)
+            runtime = 0.0
         return result
 
     if settings is None:
@@ -59,16 +56,14 @@ def minimal(definition, condition=lambda x: True, settings=None, timeout_after=1
     )
     def inner(x):
         if wrapped_condition(x):
-            result[:] = [x]
+            nonlocal result
+            result = x
             raise Found
 
-    definition.validate()
-    runtime = []
-    result = []
     try:
         inner()
     except Found:
-        return result[0]
+        return result
     raise Unsatisfiable(
         "Could not find any examples from %r that satisfied %s"
         % (definition, get_pretty_function_description(condition))
@@ -94,7 +89,7 @@ def assert_no_examples(strategy, condition=lambda _: True):
         pass
 
 
-def assert_all_examples(strategy, predicate):
+def assert_all_examples(strategy, predicate, settings=None):
     """Asserts that all examples of the given strategy match the predicate.
 
     :param strategy: Hypothesis strategy to check
@@ -102,6 +97,7 @@ def assert_all_examples(strategy, predicate):
     """
 
     @given(strategy)
+    @Settings(parent=settings, database=None)
     def assert_examples(s):
         msg = f"Found {s!r} using strategy {strategy} which does not match"
         assert predicate(s), msg
@@ -109,13 +105,37 @@ def assert_all_examples(strategy, predicate):
     assert_examples()
 
 
-def assert_can_trigger_event(strategy, predicate):
-    def test(buf):
-        data = ConjectureData.for_buffer(buf)
-        try:
-            data.draw(strategy)
-        except StopTest:
-            pass
-        return any(predicate(e) for e in data.events)
+def assert_simple_property(strategy, predicate, settings=None):
+    """Like assert_all_examples, intended as a self-documenting shortcut for simple constant
+    properties (`is`, `isinstance`, `==`, ...) that can be adequately verified in just a few
+    examples.
 
-    find_any(st.binary(), test)
+    For more thorough checking, use assert_all_examples.
+    """
+
+    assert_all_examples(
+        strategy,
+        predicate,
+        Settings(
+            parent=settings,
+            max_examples=15,
+            suppress_health_check=list(HealthCheck),
+        ),
+    )
+
+
+def check_can_generate_examples(strategy, settings=None):
+    """Tries to generate a small number of examples from the strategy, to verify that it can
+    do so without raising.
+
+    Nothing is returned, it only checks that no error is raised.
+    """
+
+    assert_simple_property(
+        strategy,
+        lambda _: True,
+        settings=Settings(
+            parent=settings,
+            phases=(Phase.generate,),
+        ),
+    )
