@@ -13,7 +13,6 @@ from warnings import filterwarnings
 
 from hypothesis import Phase, Verbosity, settings
 from hypothesis._settings import not_set
-from hypothesis.errors import NonInteractiveExampleWarning
 from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 
 
@@ -36,9 +35,6 @@ def run():
         category=UserWarning,
     )
 
-    # User-facing warning which does not apply to our own tests
-    filterwarnings("ignore", category=NonInteractiveExampleWarning)
-
     # We do a smoke test here before we mess around with settings.
     x = settings()
 
@@ -48,9 +44,9 @@ def run():
         v = getattr(x, s.name)
         # Check if it has a dynamically defined default and if so skip comparison.
         if getattr(settings, s.name).show_default:
-            assert v == s.default, "({!r} == x.{}) != (s.{} == {!r})".format(
-                v, s.name, s.name, s.default
-            )
+            assert (
+                v == s.default
+            ), f"({v!r} == x.{s.name}) != (s.{s.name} == {s.default!r})"
 
     settings.register_profile(
         "default",
