@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import pytest
 
@@ -27,37 +22,37 @@ def test_pareto_front_contains_different_interesting_reasons():
     with deterministic_PRNG():
 
         def test(data):
-            data.mark_interesting(data.draw_bits(4))
+            data.mark_interesting(data.draw_integer(0, 2**4 - 1))
 
         runner = ConjectureRunner(
             test,
             settings=settings(
                 max_examples=5000,
                 database=InMemoryExampleDatabase(),
-                suppress_health_check=HealthCheck.all(),
+                suppress_health_check=list(HealthCheck),
             ),
             database_key=b"stuff",
         )
 
         runner.run()
 
-        assert len(runner.pareto_front) == 2 ** 4
+        assert len(runner.pareto_front) == 2**4
 
 
 def test_database_contains_only_pareto_front():
     with deterministic_PRNG():
 
         def test(data):
-            data.target_observations["1"] = data.draw_bits(4)
-            data.draw_bits(64)
-            data.target_observations["2"] = data.draw_bits(8)
+            data.target_observations["1"] = data.draw_integer(0, 2**4 - 1)
+            data.draw_integer(0, 2**64 - 1)
+            data.target_observations["2"] = data.draw_integer(0, 2**8 - 1)
 
         db = InMemoryExampleDatabase()
 
         runner = ConjectureRunner(
             test,
             settings=settings(
-                max_examples=500, database=db, suppress_health_check=HealthCheck.all()
+                max_examples=500, database=db, suppress_health_check=list(HealthCheck)
             ),
             database_key=b"stuff",
         )
@@ -88,8 +83,8 @@ def test_clears_defunct_pareto_front():
     with deterministic_PRNG():
 
         def test(data):
-            data.draw_bits(8)
-            data.draw_bits(8)
+            data.draw_integer(0, 2**8 - 1)
+            data.draw_integer(0, 2**8 - 1)
 
         db = InMemoryExampleDatabase()
 
@@ -98,7 +93,7 @@ def test_clears_defunct_pareto_front():
             settings=settings(
                 max_examples=10000,
                 database=db,
-                suppress_health_check=HealthCheck.all(),
+                suppress_health_check=list(HealthCheck),
                 phases=[Phase.reuse],
             ),
             database_key=b"stuff",
@@ -116,8 +111,8 @@ def test_down_samples_the_pareto_front():
     with deterministic_PRNG():
 
         def test(data):
-            data.draw_bits(8)
-            data.draw_bits(8)
+            data.draw_integer(0, 2**8 - 1)
+            data.draw_integer(0, 2**8 - 1)
 
         db = InMemoryExampleDatabase()
 
@@ -126,7 +121,7 @@ def test_down_samples_the_pareto_front():
             settings=settings(
                 max_examples=1000,
                 database=db,
-                suppress_health_check=HealthCheck.all(),
+                suppress_health_check=list(HealthCheck),
                 phases=[Phase.reuse],
             ),
             database_key=b"stuff",
@@ -145,8 +140,8 @@ def test_stops_loading_pareto_front_if_interesting():
     with deterministic_PRNG():
 
         def test(data):
-            data.draw_bits(8)
-            data.draw_bits(8)
+            data.draw_integer(0, 2**8 - 1)
+            data.draw_integer(0, 2**8 - 1)
             data.mark_interesting()
 
         db = InMemoryExampleDatabase()
@@ -156,7 +151,7 @@ def test_stops_loading_pareto_front_if_interesting():
             settings=settings(
                 max_examples=1000,
                 database=db,
-                suppress_health_check=HealthCheck.all(),
+                suppress_health_check=list(HealthCheck),
                 phases=[Phase.reuse],
             ),
             database_key=b"stuff",
@@ -174,9 +169,9 @@ def test_uses_tags_in_calculating_pareto_front():
     with deterministic_PRNG():
 
         def test(data):
-            if data.draw_bits(1):
+            if data.draw_boolean():
                 data.start_example(11)
-                data.draw_bits(8)
+                data.draw_integer(0, 2**8 - 1)
                 data.stop_example()
 
         runner = ConjectureRunner(
@@ -193,7 +188,7 @@ def test_uses_tags_in_calculating_pareto_front():
 def test_optimises_the_pareto_front():
     def test(data):
         count = 0
-        while data.draw_bits(8):
+        while data.draw_integer(0, 2**8 - 1):
             count += 1
 
         data.target_observations[""] = min(count, 5)
@@ -215,7 +210,7 @@ def test_optimises_the_pareto_front():
 
 def test_does_not_optimise_the_pareto_front_if_interesting():
     def test(data):
-        n = data.draw_bits(8)
+        n = data.draw_integer(0, 2**8 - 1)
         data.target_observations[""] = n
         if n == 255:
             data.mark_interesting()
@@ -236,10 +231,10 @@ def test_does_not_optimise_the_pareto_front_if_interesting():
 
 
 def test_stops_optimising_once_interesting():
-    hi = 2 ** 16 - 1
+    hi = 2**16 - 1
 
     def test(data):
-        n = data.draw_bits(16)
+        n = data.draw_integer(0, 2**16 - 1)
         data.target_observations[""] = n
         if n < hi:
             data.mark_interesting()
