@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import os
 import sys
@@ -41,41 +36,35 @@ def preserving_dfas():
         dfas.SHRINKING_DFAS.update(original)
         dfas.update_learned_dfas()
     assert TEST_DFA_NAME not in dfas.SHRINKING_DFAS
-    with open(dfas.learned_dfa_file) as i:
-        assert TEST_DFA_NAME not in i.read()
+    assert TEST_DFA_NAME not in dfas.learned_dfa_file.read_text(encoding="utf-8")
 
 
 def test_updating_the_file_makes_no_changes_normally():
-    with open(dfas.learned_dfa_file) as i:
-        source1 = i.read()
+    source1 = dfas.learned_dfa_file.read_text(encoding="utf-8")
 
     dfas.update_learned_dfas()
 
-    with open(dfas.learned_dfa_file) as i:
-        source2 = i.read()
+    source2 = dfas.learned_dfa_file.read_text(encoding="utf-8")
 
     assert source1 == source2
 
 
 def test_updating_the_file_include_new_shrinkers():
     with preserving_dfas():
-        with open(dfas.learned_dfa_file) as i:
-            source1 = i.read()
+        source1 = dfas.learned_dfa_file.read_text(encoding="utf-8")
 
         dfas.SHRINKING_DFAS[TEST_DFA_NAME] = "hello"
 
         dfas.update_learned_dfas()
 
-        with open(dfas.learned_dfa_file) as i:
-            source2 = i.read()
+        source2 = dfas.learned_dfa_file.read_text(encoding="utf-8")
 
         assert source1 != source2
         assert repr(TEST_DFA_NAME) in source2
 
     assert TEST_DFA_NAME not in dfas.SHRINKING_DFAS
 
-    with open(dfas.learned_dfa_file) as i:
-        assert "test name" not in i.read()
+    assert "test name" not in dfas.learned_dfa_file.read_text(encoding="utf-8")
 
 
 def called_by_shrinker():
@@ -96,7 +85,7 @@ def a_bad_test_function():
     cache = {0: False}
 
     def test_function(data):
-        n = data.draw_bits(64)
+        n = data.draw_integer(0, 2**64 - 1)
         if n < 1000:
             return
 
@@ -142,16 +131,15 @@ def non_normalized_test_function(data):
     is hard to move between. It's basically unreasonable for
     our shrinker to be able to transform from one to the other
     because of how different they are."""
-    data.draw_bits(8)
-    if data.draw_bits(1):
-        n = data.draw_bits(10)
+    if data.draw_boolean():
+        n = data.draw_integer(0, 2**10 - 1)
         if 100 < n < 1000:
-            data.draw_bits(8)
+            data.draw_integer(0, 2**8 - 1)
             data.mark_interesting()
     else:
-        n = data.draw_bits(64)
-        if n > 10000:
-            data.draw_bits(8)
+        n = data.draw_integer(0, 2**10 - 1)
+        if n > 500:
+            data.draw_integer(0, 2**8 - 1)
             data.mark_interesting()
 
 
@@ -168,12 +156,12 @@ def test_can_learn_to_normalize_the_unnormalized():
 
 def test_will_error_on_uninteresting_test():
     with pytest.raises(AssertionError):
-        dfas.normalize(TEST_DFA_NAME, lambda data: data.draw_bits(64))
+        dfas.normalize(TEST_DFA_NAME, lambda data: data.draw_integer(0, 2**64 - 1))
 
 
 def test_makes_no_changes_if_already_normalized():
     def test_function(data):
-        if data.draw_bits(16) >= 1000:
+        if data.draw_integer(0, 2**16 - 1) >= 1000:
             data.mark_interesting()
 
     with preserving_dfas():
@@ -188,8 +176,8 @@ def test_makes_no_changes_if_already_normalized():
 
 def test_learns_to_bridge_only_two():
     def test_function(data):
-        m = data.draw_bits(8)
-        n = data.draw_bits(8)
+        m = data.draw_integer(0, 2**8 - 1)
+        n = data.draw_integer(0, 2**8 - 1)
 
         if (m, n) in ((10, 100), (2, 8)):
             data.mark_interesting()
@@ -216,7 +204,7 @@ def test_learns_to_bridge_only_two_with_overlap():
 
     def test_function(data):
         for i in range(len(u)):
-            c = data.draw_bits(8)
+            c = data.draw_integer(0, 2**8 - 1)
             if c != u[i]:
                 if c != v[i]:
                     return
@@ -224,7 +212,7 @@ def test_learns_to_bridge_only_two_with_overlap():
         else:
             data.mark_interesting()
         for j in range(i + 1, len(v)):
-            if data.draw_bits(8) != v[j]:
+            if data.draw_integer(0, 2**8 - 1) != v[j]:
                 return
 
         data.mark_interesting()
@@ -243,15 +231,15 @@ def test_learns_to_bridge_only_two_with_suffix():
     v = [0] * 10 + [7]
 
     def test_function(data):
-        n = data.draw_bits(8)
+        n = data.draw_integer(0, 2**8 - 1)
         if n == 7:
             data.mark_interesting()
         elif n != 0:
             return
         for _ in range(9):
-            if data.draw_bits(8) != 0:
+            if data.draw_integer(0, 2**8 - 1) != 0:
                 return
-        if data.draw_bits(8) == 7:
+        if data.draw_integer(0, 2**8 - 1) == 7:
             data.mark_interesting()
 
     runner = ConjectureRunner(
