@@ -1,22 +1,24 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
+
+import sys
 
 import pytest
 
-from hypothesis import assume, given, strategies as st
-from hypothesis.errors import InvalidArgument
+from hypothesis import HealthCheck, assume, given, settings, strategies as st
+from hypothesis.errors import (
+    HypothesisDeprecationWarning,
+    HypothesisWarning,
+    InvalidArgument,
+)
+
 from tests.common.debug import minimal
 from tests.common.utils import flaky
 
@@ -45,6 +47,7 @@ def draw_ordered_with_assume(draw):
 
 
 @given(draw_ordered_with_assume())
+@settings(suppress_health_check=[HealthCheck.filter_too_much])
 def test_can_assume_in_draw(xy):
     assert xy[0] < xy[1]
 
@@ -79,6 +82,14 @@ def test_errors_given_kwargs_only():
             pass
 
 
+def test_warning_given_no_drawfn_call():
+    with pytest.warns(HypothesisDeprecationWarning):
+
+        @st.composite
+        def foo(_):
+            return "bar"
+
+
 def test_can_use_pure_args():
     @st.composite
     def stuff(*args):
@@ -126,6 +137,7 @@ def test_does_not_change_arguments(data, ls):
     # regression test for issue #1017 or other argument mutation
     @st.composite
     def strat(draw, arg):
+        draw(st.none())
         return arg
 
     ex = data.draw(strat(ls))
@@ -175,3 +187,20 @@ def test_applying_composite_decorator_to_methods(data):
         x = data.draw(strategy)
         assert isinstance(x, int)
         assert 0 <= x <= 10
+
+
+def test_drawfn_cannot_be_instantiated():
+    with pytest.raises(TypeError):
+        st.DrawFn()
+
+
+@pytest.mark.skipif(sys.version_info[:2] == (3, 9), reason="stack depth varies???")
+def test_warns_on_strategy_annotation():
+    with pytest.warns(HypothesisWarning, match="Return-type annotation") as w:
+
+        @st.composite
+        def my_integers(draw: st.DrawFn) -> st.SearchStrategy[int]:
+            return draw(st.integers())
+
+    assert len(w.list) == 1
+    assert w.list[0].filename == __file__  # check stacklevel points to user code
