@@ -22,7 +22,7 @@ from hypothesis import given, settings, strategies as ds
 from hypothesis.errors import InvalidArgument
 from hypothesis.vendor.pretty import pretty
 
-from tests.common.debug import minimal
+from tests.common.debug import check_can_generate_examples, minimal
 
 # Use `pretty` instead of `repr` for building test names, so that set and dict
 # parameters print consistently across multiple worker processes with different
@@ -119,6 +119,12 @@ def fn_ktest(*fnkwargs):
     (ds.lists, {"elements": ds.integers(), "unique_by": ()}),
     (ds.lists, {"elements": ds.integers(), "unique_by": (1,)}),
     (ds.lists, {"elements": ds.sampled_from([0, 1]), "min_size": 3, "unique": True}),
+    (ds.lists, {"elements": ds.none(), "min_size": 100_000}),
+    (ds.lists, {"elements": ds.none(), "min_size": 100_000, "unique": True}),
+    (
+        ds.lists,
+        {"elements": ds.sampled_from([1, 2]), "min_size": 100_000, "unique": True},
+    ),
     (ds.text, {"min_size": 10, "max_size": 9}),
     (ds.text, {"alphabet": [1]}),
     (ds.text, {"alphabet": ["abc"]}),
@@ -128,13 +134,13 @@ def fn_ktest(*fnkwargs):
     (ds.text, {"alphabet": ds.sampled_from([123, 456])}),
     (ds.text, {"alphabet": ds.builds(lambda: "abc")}),
     (ds.text, {"alphabet": ds.builds(lambda: 123)}),
+    (ds.text, {"alphabet": "abc", "min_size": 100_000}),
     (ds.from_regex, {"regex": 123}),
     (ds.from_regex, {"regex": b"abc", "alphabet": "abc"}),
     (ds.from_regex, {"regex": b"abc", "alphabet": b"def"}),
     (ds.from_regex, {"regex": "abc", "alphabet": "def"}),
+    (ds.from_regex, {"regex": "aa|bb", "alphabet": "c"}),
     (ds.from_regex, {"regex": "[abc]", "alphabet": "def"}),
-    (ds.from_regex, {"regex": "[a-d]", "alphabet": "def"}),
-    (ds.from_regex, {"regex": "[f-z]", "alphabet": "def"}),
     (ds.from_regex, {"regex": "[ab]x[de]", "alphabet": "abcdef"}),
     (ds.from_regex, {"regex": "...", "alphabet": ds.builds(lambda: "a")}),
     (ds.from_regex, {"regex": "abc", "alphabet": ds.sampled_from("def")}),
@@ -190,13 +196,14 @@ def fn_ktest(*fnkwargs):
     (ds.characters, {"min_codepoint": "1"}),
     (ds.characters, {"max_codepoint": -1}),
     (ds.characters, {"max_codepoint": "1"}),
-    (ds.characters, {"whitelist_categories": []}),
+    (ds.characters, {"categories": []}),
+    (ds.characters, {"categories": ["Nd"], "exclude_categories": ["Nd"]}),
     (ds.characters, {"whitelist_categories": ["Nd"], "blacklist_categories": ["Nd"]}),
+    (ds.characters, {"include_characters": "a", "blacklist_characters": "b"}),
     (ds.characters, {"codec": 100}),
-    (ds.characters, {"codec": "cp861"}),  # not yet implemented
     (ds.characters, {"codec": "this is not a valid codec name"}),
-    (ds.characters, {"codec": "ascii", "whitelist_characters": "é"}),
-    (ds.characters, {"codec": "utf-8", "whitelist_categories": "Cs"}),
+    (ds.characters, {"codec": "ascii", "include_characters": "é"}),
+    (ds.characters, {"codec": "utf-8", "categories": "Cs"}),
     (ds.slices, {"size": None}),
     (ds.slices, {"size": "chips"}),
     (ds.slices, {"size": -1}),
@@ -214,7 +221,7 @@ def fn_ktest(*fnkwargs):
 )
 def test_validates_keyword_arguments(fn, kwargs):
     with pytest.raises(InvalidArgument):
-        fn(**kwargs).example()
+        check_can_generate_examples(fn(**kwargs))
 
 
 @fn_ktest(
@@ -282,12 +289,20 @@ def test_validates_keyword_arguments(fn, kwargs):
     (ds.text, {"alphabet": ds.just("a")}),
     (ds.text, {"alphabet": ds.sampled_from("abc")}),
     (ds.text, {"alphabet": ds.builds(lambda: "a")}),
-    (ds.characters, {"whitelist_categories": ["N"]}),
-    (ds.characters, {"blacklist_categories": []}),
+    (ds.characters, {"codec": "ascii"}),
+    (ds.characters, {"codec": "latin1"}),
+    (ds.characters, {"categories": ["N"]}),
+    (ds.characters, {"exclude_categories": []}),
+    (ds.characters, {"whitelist_characters": "a", "codec": "ascii"}),
+    (ds.characters, {"blacklist_characters": "a"}),
+    (ds.characters, {"whitelist_categories": ["Nd"]}),
+    (ds.characters, {"blacklist_categories": ["Nd"]}),
     (ds.from_regex, {"regex": "abc", "alphabet": "abc"}),
     (ds.from_regex, {"regex": "abc", "alphabet": "abcdef"}),
     (ds.from_regex, {"regex": "[abc]", "alphabet": "abcdef"}),
     (ds.from_regex, {"regex": "[a-f]", "alphabet": "abef"}),
+    (ds.from_regex, {"regex": "[a-d]", "alphabet": "def"}),
+    (ds.from_regex, {"regex": "[f-z]", "alphabet": "def"}),
     (ds.from_regex, {"regex": "abc", "alphabet": ds.sampled_from("abc")}),
     (ds.from_regex, {"regex": "abc", "alphabet": ds.characters(codec="ascii")}),
     (ds.ip_addresses, {}),
@@ -303,7 +318,7 @@ def test_validates_keyword_arguments(fn, kwargs):
     (ds.ip_addresses, {"v": 6, "network": IPv6Network("::/64")}),
 )
 def test_produces_valid_examples_from_keyword(fn, kwargs):
-    fn(**kwargs).example()
+    check_can_generate_examples(fn(**kwargs))
 
 
 @fn_test(
@@ -313,7 +328,7 @@ def test_produces_valid_examples_from_keyword(fn, kwargs):
 )
 def test_validates_args(fn, args):
     with pytest.raises(InvalidArgument):
-        fn(*args).example()
+        check_can_generate_examples(fn(*args))
 
 
 @fn_test(
@@ -324,23 +339,25 @@ def test_validates_args(fn, args):
     (ds.builds, (lambda x, y: x + y, ds.integers(), ds.integers())),
 )
 def test_produces_valid_examples_from_args(fn, args):
-    fn(*args).example()
+    check_can_generate_examples(fn(*args))
 
 
 def test_build_class_with_target_kwarg():
     NamedTupleWithTargetField = collections.namedtuple("Something", ["target"])
-    ds.builds(NamedTupleWithTargetField, target=ds.integers()).example()
+    check_can_generate_examples(
+        ds.builds(NamedTupleWithTargetField, target=ds.integers())
+    )
 
 
 def test_builds_raises_with_no_target():
     with pytest.raises(TypeError):
-        ds.builds().example()
+        check_can_generate_examples(ds.builds())
 
 
 @pytest.mark.parametrize("non_callable", [1, "abc", ds.integers()])
 def test_builds_raises_if_non_callable_as_target_kwarg(non_callable):
     with pytest.raises(TypeError):
-        ds.builds(target=non_callable).example()
+        check_can_generate_examples(ds.builds(target=non_callable))
 
 
 @pytest.mark.parametrize("non_callable", [1, "abc", ds.integers()])
@@ -348,7 +365,7 @@ def test_builds_raises_if_non_callable_as_first_arg(non_callable):
     # If there are any positional arguments, then the target (which must be
     # callable) must be specified as the first one.
     with pytest.raises(InvalidArgument):
-        ds.builds(non_callable, target=lambda x: x).example()
+        check_can_generate_examples(ds.builds(non_callable, target=lambda x: x))
 
 
 def test_tuples_raise_error_on_bad_kwargs():
