@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import os
 
@@ -39,16 +34,15 @@ def test_defaults_to_the_default():
 def test_can_set_homedir_and_it_will_exist(tmpdir):
     fs.set_hypothesis_home_dir(str(tmpdir.mkdir("kittens")))
     d = fs.storage_directory()
-    assert "kittens" in d
-    assert os.path.exists(d)
+    assert "kittens" in str(d)
+    assert d.exists()
 
 
 def test_will_pick_up_location_from_env(monkeypatch, tmpdir):
-    tmpdir = str(tmpdir)
-    monkeypatch.setattr(os, "environ", {"HYPOTHESIS_STORAGE_DIRECTORY": tmpdir})
+    monkeypatch.setattr(os, "environ", {"HYPOTHESIS_STORAGE_DIRECTORY": str(tmpdir)})
     assert fs.storage_directory() == tmpdir
 
 
 def test_storage_directories_are_not_created_automatically(tmpdir):
     fs.set_hypothesis_home_dir(str(tmpdir))
-    assert not os.path.exists(fs.storage_directory("badgers"))
+    assert not fs.storage_directory("badgers").exists()
