@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import re
 import sys
@@ -30,8 +25,15 @@ from hypothesis.strategies._internal.regex import (
     UNICODE_SPACE_CHARS,
     UNICODE_WORD_CATEGORIES,
     base_regex_strategy,
+    regex_strategy,
+)
+
+from tests.common.debug import (
+    assert_all_examples,
+    assert_no_examples,
+    check_can_generate_examples,
+    find_any,
 )
-from tests.common.debug import assert_all_examples, assert_no_examples, find_any
 
 
 def is_ascii(s):
@@ -67,10 +69,10 @@ def unicode_regex(pattern):
     return re.compile(pattern, re.UNICODE)
 
 
-def _test_matching_pattern(pattern, isvalidchar, is_unicode=False):
+def _test_matching_pattern(pattern, *, isvalidchar, is_unicode=False):
     r = unicode_regex(pattern) if is_unicode else ascii_regex(pattern)
 
-    codepoints = range(0, sys.maxunicode + 1) if is_unicode else range(1, 128)
+    codepoints = range(sys.maxunicode + 1) if is_unicode else range(1, 128)
     for c in [chr(x) for x in codepoints]:
         if isvalidchar(c):
             msg = "%r supposed to match %r (%r, category %r), but it doesn't"
@@ -100,7 +102,7 @@ def test_matching(category, predicate, invert, is_unicode):
     else:
         pred = predicate
 
-    _test_matching_pattern(category, pred, is_unicode)
+    _test_matching_pattern(category, isvalidchar=pred, is_unicode=is_unicode)
 
 
 @pytest.mark.parametrize(
@@ -137,12 +139,16 @@ def test_matching(category, predicate, invert, is_unicode):
         "[^\\S]",  # categories
     ],
 )
-@pytest.mark.parametrize("encode", [False, True])
+@pytest.mark.parametrize("encode", [None, False, True])
 def test_can_generate(pattern, encode):
+    alphabet = st.characters(max_codepoint=1000) if encode is None else None
     if encode:
         pattern = pattern.encode("ascii")
     with local_settings(settings(suppress_health_check=[HealthCheck.data_too_large])):
-        assert_all_examples(st.from_regex(pattern), re.compile(pattern).search)
+        assert_all_examples(
+            st.from_regex(pattern, alphabet=alphabet),
+            re.compile(pattern).search,
+        )
 
 
 @pytest.mark.parametrize(
@@ -177,16 +183,12 @@ def test_any_doesnt_generate_newline():
 
 @pytest.mark.parametrize("pattern", [re.compile("\\A.\\Z", re.DOTALL), "(?s)\\A.\\Z"])
 def test_any_with_dotall_generate_newline(pattern):
-    find_any(
-        st.from_regex(pattern), lambda s: s == "\n", settings(max_examples=10 ** 6)
-    )
+    find_any(st.from_regex(pattern), lambda s: s == "\n", settings(max_examples=10**6))
 
 
 @pytest.mark.parametrize("pattern", [re.compile(b"\\A.\\Z", re.DOTALL), b"(?s)\\A.\\Z"])
 def test_any_with_dotall_generate_newline_binary(pattern):
-    find_any(
-        st.from_regex(pattern), lambda s: s == b"\n", settings(max_examples=10 ** 6)
-    )
+    find_any(st.from_regex(pattern), lambda s: s == b"\n", settings(max_examples=10**6))
 
 
 @pytest.mark.parametrize(
@@ -211,7 +213,7 @@ def test_groups(pattern, is_unicode, invert):
         def group_pred(s):
             return not _p(s)
 
-    pattern = "^%s\\Z" % (pattern,)
+    pattern = f"^{pattern}\\Z"
 
     compiler = unicode_regex if is_unicode else ascii_regex
     strategy = st.from_regex(compiler(pattern))
@@ -261,7 +263,9 @@ def test_can_handle_boundaries_nested(s):
 def test_groupref_not_shared_between_regex():
     # If group references are (incorrectly!) shared between regex, this would
     # fail as the would only be one reference.
-    st.tuples(st.from_regex("(a)\\1"), st.from_regex("(b)\\1")).example()
+    check_can_generate_examples(
+        st.tuples(st.from_regex("(a)\\1"), st.from_regex("(b)\\1"))
+    )
 
 
 @pytest.mark.skipif(
@@ -271,8 +275,8 @@ def test_groupref_not_shared_between_regex():
 @given(st.data())
 def test_group_ref_is_not_shared_between_identical_regex(data):
     pattern = re.compile("^(.+)\\1\\Z", re.UNICODE)
-    x = data.draw(base_regex_strategy(pattern))
-    y = data.draw(base_regex_strategy(pattern))
+    x = data.draw(base_regex_strategy(pattern, alphabet=st.characters()))
+    y = data.draw(base_regex_strategy(pattern, alphabet=st.characters()))
     assume(x != y)
     assert pattern.match(x).end() == len(x)
     assert pattern.match(y).end() == len(y)
@@ -280,9 +284,11 @@ def test_group_ref_is_not_shared_between_identical_regex(data):
 
 @given(st.data())
 def test_does_not_leak_groups(data):
-    a = data.draw(base_regex_strategy(re.compile("^(a)\\Z")))
+    a = data.draw(base_regex_strategy(re.compile("^(a)\\Z"), alphabet=st.characters()))
     assert a == "a"
-    b = data.draw(base_regex_strategy(re.compile("^(?(1)a|b)(.)\\Z")))
+    b = data.draw(
+        base_regex_strategy(re.compile("^(?(1)a|b)(.)\\Z"), alphabet=st.characters())
+    )
     assert b[0] == "b"
 
 
@@ -291,7 +297,7 @@ def test_positive_lookbehind():
 
 
 def test_positive_lookahead():
-    st.from_regex("a(?=bc).*").filter(lambda s: s.startswith("abc")).example()
+    find_any(st.from_regex("a(?=bc).*"), lambda s: s.startswith("abc"))
 
 
 def test_negative_lookbehind():
@@ -424,20 +430,24 @@ def test_fullmatch_generates_example(pattern, matching_str):
     find_any(
         st.from_regex(pattern, fullmatch=True),
         lambda s: s == matching_str,
-        settings(max_examples=10 ** 6),
+        settings(max_examples=10**6),
     )
 
 
 @pytest.mark.parametrize(
     "pattern,eqiv_pattern",
     [
+        (r"", r"\A\Z"),
+        (b"", rb"\A\Z"),
+        (r"(?#comment)", r"\A\Z"),
+        (rb"(?#comment)", rb"\A\Z"),
         ("a", "\\Aa\\Z"),
         ("[Aa]", "\\A[Aa]\\Z"),
         ("[ab]*", "\\A[ab]*\\Z"),
-        (b"[Aa]", br"\A[Aa]\Z"),
-        (b"[ab]*", br"\A[ab]*\Z"),
+        (b"[Aa]", rb"\A[Aa]\Z"),
+        (b"[ab]*", rb"\A[ab]*\Z"),
         (re.compile("[ab]*", re.IGNORECASE), re.compile("\\A[ab]*\\Z", re.IGNORECASE)),
-        (re.compile(br"[ab]", re.IGNORECASE), re.compile(br"\A[ab]\Z", re.IGNORECASE)),
+        (re.compile(rb"[ab]", re.IGNORECASE), re.compile(rb"\A[ab]\Z", re.IGNORECASE)),
     ],
 )
 def test_fullmatch_matches(pattern, eqiv_pattern):
@@ -462,3 +472,17 @@ def test_sets_allow_multichar_output_in_ignorecase_mode():
         st.from_regex(re.compile("[\u0130_]", re.IGNORECASE)),
         lambda s: len(s) > 1,
     )
+
+
+def test_internals_can_disable_newline_from_dollar_for_jsonschema():
+    pattern = "^abc$"
+    find_any(st.from_regex(pattern), lambda s: s == "abc\n")
+    assert_all_examples(
+        regex_strategy(
+            pattern,
+            False,
+            alphabet=st.characters(),
+            _temp_jsonschema_hack_no_end_newline=True,
+        ),
+        lambda s: s == "abc",
+    )
