@@ -1,18 +1,14 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
+import re
 import time
 import traceback
 
@@ -24,6 +20,7 @@ from hypothesis import (
     event,
     example,
     given,
+    reject,
     settings,
     stateful,
     strategies as st,
@@ -49,7 +46,7 @@ def unique_events(stats):
 
 def test_notes_hard_to_satisfy():
     @given(st.integers())
-    @settings(suppress_health_check=HealthCheck.all())
+    @settings(suppress_health_check=list(HealthCheck))
     def test(i):
         assume(i == 13)
 
@@ -84,7 +81,7 @@ class Foo:
         seen.append(self)
         global counter
         counter += 1
-        return "COUNTER %d" % (counter,)
+        return f"COUNTER {counter}"
 
 
 def test_formats_are_evaluated_only_once():
@@ -142,7 +139,7 @@ def test_flaky_exit():
             if first[0]:
                 first[0] = False
                 print("Hi")
-                assert False
+                raise AssertionError
 
     stats = call_for_statistics(test)
     assert stats["stopped-because"] == "test was flaky"
@@ -150,13 +147,14 @@ def test_flaky_exit():
 
 @pytest.mark.parametrize("draw_delay", [False, True])
 @pytest.mark.parametrize("test_delay", [False, True])
-def test_draw_time_percentage(draw_delay, test_delay):
+def test_draw_timing(draw_delay, test_delay):
     time.freeze()
 
     @st.composite
     def s(draw):
         if draw_delay:
             time.sleep(0.05)
+        draw(st.integers())
 
     @given(s())
     def test(_):
@@ -165,11 +163,10 @@ def test_draw_time_percentage(draw_delay, test_delay):
 
     stats = describe_statistics(call_for_statistics(test))
     if not draw_delay:
-        assert "~ 0%" in stats
-    elif test_delay:
-        assert "~ 50%" in stats
+        assert "< 1ms" in stats
     else:
-        assert "~ 100%" in stats
+        match = re.search(r"of which ~ (?P<gentime>\d+)", stats)
+        assert 49 <= int(match.group("gentime")) <= 51
 
 
 def test_has_lambdas_in_output():
@@ -251,7 +248,7 @@ def test_statistics_for_threshold_problem():
 
 
 def test_statistics_with_events_and_target():
-    @given(st.sampled_from("1234"))
+    @given(st.integers(0, 10_000))
     def test(value):
         event(value)
         target(float(value), label="a target")
@@ -259,3 +256,33 @@ def test_statistics_with_events_and_target():
     stats = describe_statistics(call_for_statistics(test))
     assert "- Events:" in stats
     assert "- Highest target score: " in stats
+
+
+@given(st.booleans())
+def test_event_with_non_weakrefable_keys(b):
+    event((b,))
+
+
+def test_assume_adds_event_with_function_origin():
+    @given(st.integers())
+    def very_distinguishable_name(n):
+        assume(n > 100)
+
+    stats = call_for_statistics(very_distinguishable_name)
+
+    for tc in stats["generate-phase"]["test-cases"]:
+        for e in tc["events"]:
+            assert "failed to satisfy assume() in very_distinguishable_name" in e
+
+
+def test_reject_adds_event_with_function_origin():
+    @given(st.integers())
+    def very_distinguishable_name(n):
+        if n > 100:
+            reject()
+
+    stats = call_for_statistics(very_distinguishable_name)
+
+    for tc in stats["generate-phase"]["test-cases"]:
+        for e in tc["events"]:
+            assert "reject() in very_distinguishable_name" in e
