@@ -36,6 +36,7 @@ from tests.common.utils import (
     fails_with,
     no_shrink,
     raises,
+    skipif_emscripten,
 )
 
 # This particular test file is run under both pytest and nose, so it can't
@@ -114,7 +115,7 @@ class TestCases:
         assert isinstance(self, TestCases)
 
     @given(x=integers())
-    def test_abs_non_negative_varargs_kwargs_only(*args, **kw):  # noqa: B902
+    def test_abs_non_negative_varargs_kwargs_only(*args, **kw):
         assert abs(kw["x"]) >= 0
         assert isinstance(args[0], TestCases)
 
@@ -164,7 +165,7 @@ def test_does_not_catch_interrupt_during_falsify():
     def flaky_base_exception(x):
         if not calls[0]:
             calls[0] = 1
-            raise KeyboardInterrupt()
+            raise KeyboardInterrupt
 
     with raises(KeyboardInterrupt):
         flaky_base_exception()
@@ -331,12 +332,13 @@ def test_can_run_without_database():
         test_blah()
 
 
+@skipif_emscripten
 def test_can_run_with_database_in_thread():
     results = []
 
     @given(integers())
     def test_blah(x):
-        raise ValueError()
+        raise ValueError
 
     def run_test():
         try:
@@ -464,7 +466,7 @@ def test_prints_notes_once_on_failure():
     def test(xs):
         note("Hi there")
         if sum(xs) <= 100:
-            raise ValueError()
+            raise ValueError
 
     with raises(ValueError) as err:
         test()
@@ -479,4 +481,5 @@ def test_empty_lists(xs):
 def test_given_usable_inline_on_lambdas():
     xs = []
     given(booleans())(lambda x: xs.append(x))()
-    assert len(xs) == 2 and set(xs) == {False, True}
+    assert len(xs) == 2
+    assert set(xs) == {False, True}
