@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import datetime as dt
 from uuid import UUID
@@ -19,13 +14,9 @@ from uuid import UUID
 from django.conf import settings as django_settings
 from django.contrib.auth.models import User
 
-from hypothesis import HealthCheck, assume, given, infer, settings
-from hypothesis.control import reject
-from hypothesis.errors import (
-    HypothesisDeprecationWarning,
-    HypothesisException,
-    InvalidArgument,
-)
+from hypothesis import HealthCheck, assume, given, settings
+from hypothesis.control import current_build_context, reject
+from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis.extra.django import (
     TestCase,
     TransactionTestCase,
@@ -34,6 +25,8 @@ from hypothesis.extra.django import (
 )
 from hypothesis.internal.conjecture.data import ConjectureData
 from hypothesis.strategies import binary, just, lists
+
+from tests.common.debug import check_can_generate_examples
 from tests.django.toystore.models import (
     Car,
     Company,
@@ -108,21 +101,24 @@ class TestGetsBasicModels(TestCase):
         assert x.customish == "a"
 
     def test_mandatory_fields_are_mandatory(self):
-        self.assertRaises(InvalidArgument, from_model(Store).example)
+        with self.assertRaises(InvalidArgument):
+            check_can_generate_examples(from_model(Store))
 
     def test_mandatory_computed_fields_are_mandatory(self):
         with self.assertRaises(InvalidArgument):
-            from_model(MandatoryComputed).example()
+            check_can_generate_examples(from_model(MandatoryComputed))
 
     def test_mandatory_computed_fields_may_not_be_provided(self):
-        mc = from_model(MandatoryComputed, company=from_model(Company))
-        self.assertRaises(RuntimeError, mc.example)
+        with self.assertRaises(RuntimeError):
+            check_can_generate_examples(
+                from_model(MandatoryComputed, company=from_model(Company))
+            )
 
-    @given(from_model(CustomishDefault, customish=infer))
+    @given(from_model(CustomishDefault, customish=...))
     def test_customish_default_overridden_by_infer(self, x):
         assert x.customish == "a"
 
-    @given(from_model(CustomishDefault, customish=infer))
+    @given(from_model(CustomishDefault, customish=...))
     def test_customish_infer_uses_registered_instead_of_default(self, x):
         assert x.customish == "a"
 
@@ -155,20 +151,23 @@ class TestGetsBasicModels(TestCase):
         strategy = from_model(
             CompanyExtension, company=company_strategy, self_modifying=just(2)
         )
+        context = current_build_context()
         try:
-            ConjectureData.for_buffer(buf).draw(strategy)
+            context.data = ConjectureData.for_buffer(buf)
+            context.data.draw(strategy)
         except HypothesisException:
             reject()
         # Draw again with the same buffer. This will cause a duplicate
         # primary key.
-        ConjectureData.for_buffer(buf).draw(strategy)
+        context.data = ConjectureData.for_buffer(buf)
+        context.data.draw(strategy)
         assert CompanyExtension.objects.all().count() == 1
 
 
 class TestsNeedingRollback(TransactionTestCase):
     def test_can_get_examples(self):
         for _ in range(200):
-            from_model(Company).example()
+            check_can_generate_examples(from_model(Company))
 
 
 class TestRestrictedFields(TestCase):
@@ -199,9 +198,7 @@ class TestPosOnlyArg(TestCase):
     def test_user_issue_2369_regression(self, val):
         pass
 
-    def test_from_model_argspec(self):
-        self.assertRaises(TypeError, from_model().example)
-        self.assertRaises(TypeError, from_model(Car, None).example)
-        self.assertWarns(
-            HypothesisDeprecationWarning, from_model(model=Customer).example
-        )
+    def test_from_model_signature(self):
+        self.assertRaises(TypeError, from_model)
+        self.assertRaises(TypeError, from_model, Car, None)
+        self.assertRaises(TypeError, from_model, model=Customer)
