@@ -45,8 +45,8 @@ def update_recorded_outputs(request):
 def get_recorded(name, actual=""):
     file_ = pathlib.Path(__file__).parent / "recorded" / f"{name}.txt"
     if actual:
-        file_.write_text(actual)
-    return file_.read_text()
+        file_.write_text(actual, encoding="utf-8")
+    return file_.read_text(encoding="utf-8")
 
 
 def timsort(seq: Sequence[int]) -> Sequence[int]:
@@ -186,6 +186,8 @@ def sequence_from_collections(items: CollectionsSequence[int]) -> int:
             ghostwriter.equivalent(sorted, sorted, sorted, annotate=True),
         ),
         ("addition_op_magic", ghostwriter.magic(add)),
+        ("multiplication_magic", ghostwriter.magic(operator.mul)),
+        ("matmul_magic", ghostwriter.magic(operator.matmul)),
         ("addition_op_multimagic", ghostwriter.magic(add, operator.add, numpy.add)),
         ("division_fuzz_error_handler", ghostwriter.fuzz(divide)),
         (
