@@ -223,7 +223,7 @@ def test_ghostwriter_unittest_style(func, ex):
     assert issubclass(get_test_function(source_code), unittest.TestCase)
 
 
-def no_annotations(foo=None, bar=False):
+def no_annotations(foo=None, *, bar=False):
     pass
 
 
@@ -404,7 +404,7 @@ def test_unrepr_identity_elem():
     # we can walk the strategy and collect all the objects to import.
     [
         # Lazy from_type() is handled without being unwrapped
-        (LazyStrategy(from_type, (enum.Enum,), {}), {("enum", "Enum")}),
+        (LazyStrategy(from_type, (enum.Enum,), {}), {"enum"}),
         # Mapped, filtered, and flatmapped check both sides of the method
         (
             builds(enum.Enum).map(Decimal),
@@ -463,7 +463,8 @@ def temp_script_file():
             def say_hello():
                 print("Hello world!")
             """
-        )
+        ),
+        encoding="utf-8",
     )
     yield p
     p.unlink()
@@ -483,7 +484,8 @@ def temp_script_file_with_py_function():
             def py():
                 print('A function named "py" has been called')
             """
-        )
+        ),
+        encoding="utf-8",
     )
     yield p
     p.unlink()
