@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from hypothesis import given, settings, strategies as st
 from hypothesis.database import InMemoryExampleDatabase
@@ -19,6 +14,7 @@ from hypothesis.internal.compat import int_from_bytes
 from hypothesis.internal.conjecture.data import ConjectureData
 from hypothesis.internal.conjecture.engine import ConjectureRunner
 from hypothesis.internal.conjecture.shrinker import Shrinker, block_program
+
 from tests.common.utils import counts_calls, non_covering_examples
 from tests.conjecture.common import run_to_buffer, shrinking_from
 
@@ -89,15 +85,20 @@ def test_regression_1():
     # specific exception inside one of the shrink passes. It's unclear how
     # useful this regression test really is, but nothing else caught the
     # problem.
+    #
+    # update 2024-01-15: we've since changed generation and are about to
+    # change shrinking, so it's unclear if the failure case this test was aimed
+    # at (1) is still being covered or (2) even exists anymore.
+    # we can probably safely remove this once the shrinker is rebuilt.
     @run_to_buffer
     def x(data):
-        data.write(b"\x01\x02")
-        data.write(b"\x01\x00")
-        v = data.draw_bits(41)
+        data.draw_bytes(2, forced=b"\x01\x02")
+        data.draw_bytes(2, forced=b"\x01\x00")
+        v = data.draw_integer(0, 2**41 - 1)
         if v >= 512 or v == 254:
             data.mark_interesting()
 
-    assert list(x)[:-2] == [1, 2, 1, 0, 0, 0, 0, 0]
+    assert list(x)[:-2] == [1, 2, 1, 0, 0, 0, 0, 0, 0]
 
     assert int_from_bytes(x[-2:]) in (254, 512)
 
@@ -105,7 +106,7 @@ def test_regression_1():
 @given(st.integers(0, 255), st.integers(0, 255))
 def test_cached_with_masked_byte_agrees_with_results(byte_a, byte_b):
     def f(data):
-        data.draw_bits(2)
+        data.draw_integer(0, 3)
 
     runner = ConjectureRunner(f)
 
@@ -129,7 +130,7 @@ def test_block_programs_fail_efficiently(monkeypatch):
     def shrinker(data):
         values = set()
         for _ in range(256):
-            v = data.draw_bits(8)
+            v = data.draw_integer(0, 2**8 - 1)
             values.add(v)
         if len(values) == 256:
             data.mark_interesting()
