@@ -1,24 +1,23 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
-
-import os.path
 
 from hypothesis import assume, core, find, given, settings, strategies as st
-from hypothesis.database import ExampleDatabase, InMemoryExampleDatabase
+from hypothesis.database import (
+    ExampleDatabase,
+    GitHubArtifactDatabase,
+    InMemoryExampleDatabase,
+    ReadOnlyDatabase,
+)
 from hypothesis.errors import NoSuchExample, Unsatisfiable
 from hypothesis.internal.entropy import deterministic_PRNG
+
 from tests.common.utils import all_values, non_covering_examples
 
 
@@ -31,7 +30,7 @@ def test_saves_incremental_steps_in_database():
     database = InMemoryExampleDatabase()
     find(
         st.binary(min_size=10),
-        lambda x: has_a_non_zero_byte(x),
+        has_a_non_zero_byte,
         settings=settings(database=database),
         database_key=key,
     )
@@ -67,30 +66,41 @@ def test_clears_out_database_as_things_get_boring():
         if not keys:
             break
     else:
-        assert False
+        raise AssertionError
 
 
 def test_trashes_invalid_examples():
     key = b"a database key"
     database = InMemoryExampleDatabase()
-    finicky = False
+
+    invalid = set()
 
     def stuff():
         try:
-            find(
-                st.binary(min_size=100),
-                lambda x: assume(not finicky) and has_a_non_zero_byte(x),
+
+            def condition(x):
+                assume(x not in invalid)
+                return not invalid and has_a_non_zero_byte(x)
+
+            return find(
+                st.binary(min_size=5),
+                condition,
                 settings=settings(database=database),
                 database_key=key,
             )
-        except Unsatisfiable:
+        except (Unsatisfiable, NoSuchExample):
             pass
 
-    stuff()
+    with deterministic_PRNG():
+        value = stuff()
+
     original = len(all_values(database))
     assert original > 1
-    finicky = True
-    stuff()
+
+    invalid.add(value)
+    with deterministic_PRNG():
+        stuff()
+
     assert len(all_values(database)) < original
 
 
@@ -127,7 +137,7 @@ def test_respects_max_examples_in_database_usage():
 def test_does_not_use_database_when_seed_is_forced(monkeypatch):
     monkeypatch.setattr(core, "global_force_seed", 42)
     database = InMemoryExampleDatabase()
-    database.fetch = None
+    database.fetch = None  # type: ignore
 
     @settings(database=database)
     @given(st.integers())
@@ -140,10 +150,17 @@ def test_does_not_use_database_when_seed_is_forced(monkeypatch):
 @given(st.binary(), st.binary())
 def test_database_not_created_when_not_used(tmp_path_factory, key, value):
     path = tmp_path_factory.mktemp("hypothesis") / "examples"
-    assert not os.path.exists(str(path))
+    assert not path.exists()
     database = ExampleDatabase(path)
     assert not list(database.fetch(key))
-    assert not os.path.exists(str(path))
+    assert not path.exists()
     database.save(key, value)
-    assert os.path.exists(str(path))
+    assert path.exists()
     assert list(database.fetch(key)) == [value]
+
+
+def test_ga_database_not_created_when_not_used(tmp_path_factory):
+    path = tmp_path_factory.mktemp("hypothesis") / "github-actions"
+    assert not path.exists()
+    ReadOnlyDatabase(GitHubArtifactDatabase("mock", "mock", path=path))
+    assert not path.exists()
