@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import random
 
@@ -50,7 +45,7 @@ class Write:
 @attr.s()
 class Branch:
     bits = attr.ib()
-    children = attr.ib(default=attr.Factory(dict))
+    children = attr.ib(factory=dict)
 
 
 @attr.s()
@@ -75,6 +70,12 @@ branches = st.builds(Branch, bits=st.integers(1, 64))
 writes = st.builds(Write, value=st.binary(min_size=1), child=nodes)
 
 
+# Remember what the default phases are with no test running, so that we can
+# run an outer test with non-default phases and then restore the defaults for
+# the inner test.
+_default_phases = settings.default.phases
+
+
 def run_language_test_for(root, data, seed):
     random.seed(seed)
 
@@ -82,11 +83,11 @@ def run_language_test_for(root, data, seed):
         node = root
         while not isinstance(node, Terminal):
             if isinstance(node, Write):
-                local_data.write(node.value)
+                local_data.draw_bytes(len(node.value), forced=node.value)
                 node = node.child
             else:
                 assert isinstance(node, Branch)
-                c = local_data.draw_bits(node.bits)
+                c = local_data.draw_integer(0, 2**node.bits - 1)
                 try:
                     node = node.children[c]
                 except KeyError:
@@ -104,9 +105,11 @@ def run_language_test_for(root, data, seed):
         settings=settings(
             max_examples=1,
             database=None,
-            suppress_health_check=HealthCheck.all(),
+            suppress_health_check=list(HealthCheck),
             verbosity=Verbosity.quiet,
-            phases=list(Phase),
+            # Restore the global default phases, so that we don't inherit the
+            # phases setting from the outer test.
+            phases=_default_phases,
         ),
     )
     try:
@@ -118,14 +121,14 @@ def run_language_test_for(root, data, seed):
 
 
 @settings(
-    suppress_health_check=HealthCheck.all(),
+    suppress_health_check=list(HealthCheck),
     deadline=None,
-    phases=set(Phase) - {Phase.shrink},
+    phases=set(settings.default.phases) - {Phase.shrink},
 )
 @given(st.data())
 def test_explore_an_arbitrary_language(data):
     root = data.draw(writes | branches)
-    seed = data.draw(st.integers(0, 2 ** 64 - 1))
+    seed = data.draw(st.integers(0, 2**64 - 1))
     run_language_test_for(root, data, seed)
 
 
