@@ -1,30 +1,24 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from random import Random
 
 import pytest
 
 from hypothesis import HealthCheck, settings
-from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.conjecture.engine import ConjectureData, ConjectureRunner
 from hypothesis.strategies._internal import SearchStrategy
 
 POISON = "POISON"
 
-MAX_INT = 2 ** 32 - 1
+MAX_INT = 2**32 - 1
 
 
 class PoisonedTree(SearchStrategy):
@@ -35,30 +29,32 @@ class PoisonedTree(SearchStrategy):
     """
 
     def __init__(self, p):
-        SearchStrategy.__init__(self)
+        super().__init__()
         self.__p = p
 
     def do_draw(self, data):
-        if cu.biased_coin(data, self.__p):
+        if data.draw_boolean(self.__p):
             return data.draw(self) + data.draw(self)
         else:
             # We draw n as two separate calls so that it doesn't show up as a
             # single block. If it did, the heuristics that allow us to move
             # blocks around would fire and it would move right, which would
             # then allow us to shrink it more easily.
-            n = (data.draw_bits(16) << 16) | data.draw_bits(16)
+            n1 = data.draw_integer(0, 2**16 - 1) << 16
+            n2 = data.draw_integer(0, 2**16 - 1)
+            n = n1 | n2
             if n == MAX_INT:
                 return (POISON,)
             else:
                 return (None,)
 
 
-LOTS = 10 ** 6
+LOTS = 10**6
 
 
 TEST_SETTINGS = settings(
     database=None,
-    suppress_health_check=HealthCheck.all(),
+    suppress_health_check=list(HealthCheck),
     max_examples=LOTS,
     deadline=None,
 )
@@ -97,13 +93,13 @@ def test_can_reduce_poison_from_any_subtree(size, seed):
     starts = [b.start for b in data.blocks if b.length == 2]
     assert len(starts) % 2 == 0
 
+    marker = bytes([1, 2, 3, 4])
     for i in range(0, len(starts), 2):
         # Now for each leaf position in the tree we try inserting a poison
         # value artificially. Additionally, we add a marker to the end that
         # must be preserved. The marker means that we are not allow to rely on
         # discarding the end of the buffer to get the desired shrink.
         u = starts[i]
-        marker = bytes([1, 2, 3, 4])
 
         def test_function_with_poison(data):
             v = data.draw(strat)
