@@ -40,7 +40,7 @@ def task(if_changed=()):
         def wrapped(*args, **kwargs):
             if if_changed and tools.IS_PULL_REQUEST:
                 if not tools.has_changes(if_changed + BUILD_FILES):
-                    changed = ", ".join(if_changed)
+                    changed = ", ".join(map(str, if_changed))
                     print(f"Skipping task due to no changes in {changed}")
                     return
             fn(*args, **kwargs)
@@ -297,7 +297,7 @@ def update_python_versions():
     pip_tool("shed", str(thisfile))
 
     # Automatically sync ci_version with the version in build.sh
-    build_sh = pathlib.Path(tools.ROOT) / "build.sh"
+    build_sh = tools.ROOT / "build.sh"
     sh_before = build_sh.read_text(encoding="utf-8")
     sh_after = re.sub(r"3\.\d\d?\.\d\d?", best[ci_version], sh_before)
     if sh_before != sh_after:
@@ -306,6 +306,33 @@ def update_python_versions():
         build_sh.chmod(0o755)
 
 
+def update_pyodide_versions():
+    vers_re = r"(\d+\.\d+\.\d+)"
+    pyodide_version = max(
+        # Don't just pick the most recent version; find the highest stable version.
+        re.findall(
+            f"pyodide_build-{vers_re}-py3-none-any.whl",  # excludes pre-releases
+            requests.get("https://pypi.org/simple/pyodide-build/").text,
+        ),
+        key=lambda version: tuple(int(x) for x in version.split(".")),
+    )
+    makefile_url = f"https://raw.githubusercontent.com/pyodide/pyodide/{pyodide_version}/Makefile.envs"
+    python_version, emscripten_version = re.search(
+        rf"export PYVERSION \?= {vers_re}\nexport PYODIDE_EMSCRIPTEN_VERSION \?= {vers_re}\n",
+        requests.get(makefile_url).text,
+    ).groups()
+
+    ci_file = tools.ROOT / ".github/workflows/main.yml"
+    config = ci_file.read_text(encoding="utf-8")
+    for name, var in [
+        ("PYODIDE", pyodide_version),
+        ("PYTHON", python_version),
+        ("EMSCRIPTEN", emscripten_version),
+    ]:
+        config = re.sub(f"{name}_VERSION: {vers_re}", f"{name}_VERSION: {var}", config)
+    ci_file.write_text(config, encoding="utf-8")
+
+
 def update_vendored_files():
     vendor = pathlib.Path(hp.PYTHON_SRC) / "hypothesis" / "vendor"
 
@@ -317,7 +344,8 @@ def update_vendored_files():
     if fname.read_bytes().splitlines()[1:] != new.splitlines()[1:]:
         fname.write_bytes(new)
 
-    # Always require the latest version of the tzdata package
+    # Always require the most recent version of tzdata - we don't need to worry about
+    # pre-releases because tzdata is a 'latest data' package  (unlike pyodide-build).
     tz_url = "https://pypi.org/pypi/tzdata/json"
     tzdata_version = requests.get(tz_url).json()["info"]["version"]
     setup = pathlib.Path(hp.BASE_DIR, "setup.py")
@@ -344,6 +372,7 @@ def upgrade_requirements():
         with open(hp.RELEASE_FILE, mode="w", encoding="utf-8") as f:
             f.write(f"RELEASE_TYPE: patch\n\n{msg}")
     update_python_versions()
+    update_pyodide_versions()
     subprocess.call(["git", "add", "."], cwd=tools.ROOT)
 
 
@@ -365,6 +394,16 @@ def documentation():
         )
 
 
+@task()
+def website():
+    subprocess.call(["pelican"], cwd=tools.ROOT / "website")
+
+
+@task()
+def live_website():
+    subprocess.call(["pelican", "--autoreload", "--listen"], cwd=tools.ROOT / "website")
+
+
 def run_tox(task, version, *args):
     python = install.python_executable(version)
 
@@ -392,12 +431,12 @@ PYTHONS = {
     "3.8": "3.8.18",
     "3.9": "3.9.18",
     "3.10": "3.10.13",
-    "3.11": "3.11.5",
-    "3.12": "3.12-dev",
-    "3.13": "3.13-dev",
+    "3.11": "3.11.8",
+    "3.12": "3.12.2",
+    "3.13": "3.13.0a3",
     "pypy3.8": "pypy3.8-7.3.11",
-    "pypy3.9": "pypy3.9-7.3.12",
-    "pypy3.10": "pypy3.10-7.3.12",
+    "pypy3.9": "pypy3.9-7.3.15",
+    "pypy3.10": "pypy3.10-7.3.15",
 }
 ci_version = "3.10"  # Keep this in sync with GH Actions main.yml and .readthedocs.yml
 
@@ -533,7 +572,7 @@ def audit_rust_in_ruby():
 
 @task()
 def python(*args):
-    os.execv(sys.executable, (sys.executable,) + args)
+    os.execv(sys.executable, (sys.executable, *args))
 
 
 @task()
