@@ -60,7 +60,7 @@ def extract_assignment_from_string(contents, name):
 
 
 def extract_assignment(filename, name):
-    with open(filename) as i:
+    with open(filename, encoding="utf-8") as i:
         return extract_assignment_from_string(i.read(), name)
 
 
@@ -93,10 +93,10 @@ def replace_assignment(filename, name, value):
     the file format. The existing value is simply the rest of the line after
     the last space after the equals.
     """
-    with open(filename) as i:
+    with open(filename, encoding="utf-8") as i:
         contents = i.read()
     result = replace_assignment_in_string(contents, name, value)
-    with open(filename, "w") as o:
+    with open(filename, "w", encoding="utf-8") as o:
         o.write(result)
 
 
@@ -112,7 +112,7 @@ VALID_RELEASE_TYPES = (MAJOR, MINOR, PATCH)
 
 
 def parse_release_file(filename):
-    with open(filename) as i:
+    with open(filename, encoding="utf-8") as i:
         return parse_release_file_contents(i.read(), filename)
 
 
@@ -150,12 +150,12 @@ def bump_version_info(version_info, release_type):
 
 
 def update_markdown_changelog(changelog, name, version, entry):
-    with open(changelog) as i:
+    with open(changelog, encoding="utf-8") as i:
         prev_contents = i.read()
 
     title = f"# {name} {version} ({release_date_string()})\n\n"
 
-    with open(changelog, "w") as o:
+    with open(changelog, "w", encoding="utf-8") as o:
         o.write(title)
         o.write(entry.strip())
         o.write("\n\n")
