@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import os
 
@@ -33,8 +28,9 @@ ALL_RST = [
 
 
 def test_passes_rst_lint():
-    pip_tool("rst-lint", *[f for f in ALL_RST if not is_sphinx(f)])
+    pip_tool("rst-lint", *(f for f in ALL_RST if not is_sphinx(f)))
 
 
-def test_passes_flake8():
+def disabled_test_passes_flake8():
+    # TODO: get these whitespace checks without flake8?
     pip_tool("flake8", "--select=W191,W291,W292,W293,W391", *ALL_RST)
